from typing import Any, Optional, List, Dict, Tuple, Callable, Union

r"""Functions that deal with names.

A non-tail address of the program may have a name. Tail addresses (i.e. the addresses in the middle of an instruction or data item) cannot have names. 
    
"""

class NearestName:
    r"""
    Utility class to help find the nearest name in a given ea/name dictionary
    
    """
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getitem__(self, index: Any) -> Any:
        r"""Returns the tupple (ea, name, index)"""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, ea_names: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __iter__(self) -> Any:
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def find(self, ea: Any) -> Any:
        r"""
        Returns a tupple (ea, name, pos) that is the nearest to the passed ea
        If no name is matched then None is returned
        
        """
        ...
    def update(self, ea_names: Any) -> Any:
        r"""Updates the ea/names map"""
        ...

class ea_name_t:
    @property
    def ea(self) -> Any: ...
    @property
    def name(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...

class ea_name_vec_t:
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getitem__(self, i: size_t) -> ea_name_t:
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __iter__(self) -> Any:
        r"""Helper function, to be set as __iter__ method for qvector-, or array-based classes."""
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __len__(self) -> int:
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __setitem__(self, i: size_t, v: ea_name_t) -> None:
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def append(self, x: ea_name_t) -> None:
        ...
    def at(self, _idx: size_t) -> ea_name_t:
        ...
    def back(self) -> Any:
        ...
    def begin(self, args: Any) -> const_iterator:
        ...
    def capacity(self) -> int:
        ...
    def clear(self) -> None:
        ...
    def empty(self) -> bool:
        ...
    def end(self, args: Any) -> const_iterator:
        ...
    def erase(self, args: Any) -> iterator:
        ...
    def extend(self, x: ea_name_vec_t) -> None:
        ...
    def extract(self) -> ea_name_t:
        ...
    def front(self) -> Any:
        ...
    def grow(self, args: Any) -> None:
        ...
    def inject(self, s: ea_name_t, len: size_t) -> None:
        ...
    def insert(self, it: ea_name_t, x: ea_name_t) -> iterator:
        ...
    def pop_back(self) -> None:
        ...
    def push_back(self, args: Any) -> ea_name_t:
        ...
    def qclear(self) -> None:
        ...
    def reserve(self, cnt: size_t) -> None:
        ...
    def resize(self, args: Any) -> None:
        ...
    def size(self) -> int:
        ...
    def swap(self, r: ea_name_vec_t) -> None:
        ...
    def truncate(self) -> None:
        ...

def append_struct_fields(disp: adiff_t, n: int, path: tid_t, flags: flags64_t, delta: adiff_t, appzero: bool) -> str:
    r"""Append names of struct fields to a name if the name is a struct name. 
            
    :param disp: displacement from the name
    :param n: operand number in which the name appears
    :param path: path in the struct. path is an array of id's. maximal length of array is MAXSTRUCPATH. the first element of the array is the structure id. consecutive elements are id's of used union members (if any).
    :param flags: the input flags. they will be returned if the struct cannot be found.
    :param delta: delta to add to displacement
    :param appzero: should append a struct field name if the displacement is zero?
    :returns: flags of the innermost struct member or the input flags
    """
    ...

def calc_gtn_flags(fromaddr: Any, ea: Any) -> Any:
    r"""
    Calculate flags for get_ea_name() function
    
    :param fromaddr: the referring address. May be BADADDR.
    :param ea: linear address
    
    :returns: flags
    
    """
    ...

def cleanup_name(ea: ida_idaapi.ea_t, name: str, flags: int = 0) -> str:
    ...

def del_debug_names(ea1: ida_idaapi.ea_t, ea2: ida_idaapi.ea_t) -> None:
    ...

def del_global_name(ea: ida_idaapi.ea_t) -> bool:
    ...

def del_local_name(ea: ida_idaapi.ea_t) -> bool:
    ...

def demangle_name(name: str, disable_mask: int, demreq: demreq_type_t = 2) -> str:
    r"""Demangle a name. 
            
    :param name: name to demangle
    :param disable_mask: bits to inhibit parts of demangled name (see MNG_). by the M_COMPILER bits a specific compiler can be selected (see MT_).
    :param demreq: the request type demreq_type_t
    :returns: ME_... or MT__ bitmasks from demangle.hpp
    """
    ...

def extract_name(line: str, x: int) -> str:
    r"""Extract a name or address from the specified string. 
            
    :param line: input string
    :param x: x coordinate of cursor
    :returns: -1 if cannot extract. otherwise length of the name
    """
    ...

def force_name(ea: ida_idaapi.ea_t, name: str, flags: int = 0) -> bool:
    ...

def get_colored_demangled_name(ea: ida_idaapi.ea_t, inhibitor: int, demform: int, gtn_flags: int = 0) -> str:
    ...

def get_colored_long_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    ...

def get_colored_name(ea: ida_idaapi.ea_t) -> str:
    ...

def get_colored_short_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    ...

def get_cp_validity(args: Any) -> bool:
    r"""Is the given codepoint (or range) acceptable in the given context? If 'endcp' is not BADCP, it is considered to be the end of the range: [cp, endcp), and is not included in the range 
            
    """
    ...

def get_debug_name(ea_ptr: ea_t, how: debug_name_how_t) -> str:
    ...

def get_debug_name_ea(name: str) -> ida_idaapi.ea_t:
    ...

def get_debug_names(args: Any) -> Any:
    ...

def get_demangled_name(ea: ida_idaapi.ea_t, inhibitor: int, demform: int, gtn_flags: int = 0) -> str:
    ...

def get_ea_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    r"""Get name at the specified address. 
            
    :param ea: linear address
    :param gtn_flags: how exactly the name should be retrieved. combination of bits for get_ea_name() function. There is a convenience bits
    :returns: success
    """
    ...

def get_long_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    ...

def get_mangled_name_type(name: str) -> mangled_name_type_t:
    ...

def get_name(ea: ida_idaapi.ea_t) -> str:
    ...

def get_name_base_ea(_from: ida_idaapi.ea_t, to: ida_idaapi.ea_t) -> ida_idaapi.ea_t:
    r"""Get address of the name used in the expression for the address 
            
    :param to: the referenced address
    :returns: address of the name used to represent the operand
    """
    ...

def get_name_color(_from: ida_idaapi.ea_t, ea: ida_idaapi.ea_t) -> color_t:
    r"""Calculate flags for get_ea_name() function.
    
    Get name color. 
            
    :param ea: linear address
    """
    ...

def get_name_ea(_from: ida_idaapi.ea_t, name: str) -> ida_idaapi.ea_t:
    r"""Get the address of a name. This function resolves a name into an address. It can handle regular global and local names, as well as debugger names. 
            
    :param name: any name in the program or nullptr
    :returns: address of the name or BADADDR
    """
    ...

def get_name_expr(_from: ida_idaapi.ea_t, n: int, ea: ida_idaapi.ea_t, off: int, flags: int = 1) -> str:
    r"""Convert address to name expression (name with a displacement). This function takes into account fixup information and returns a colored name expression (in the form <name> +/- <offset>). It also knows about structure members and arrays. If the specified address doesn't have a name, a dummy name is generated. 
            
    :param n: number of referencing operand. for data items specify 0
    :param ea: address to convert to name expression
    :param off: the value of name expression. this parameter is used only to check that the name expression will have the wanted value. 'off' may be equal to BADADDR but this is discouraged because it prohibits checks.
    :param flags: Name expression flags
    :returns: < 0 if address is not valid, no segment or other failure. otherwise the length of the name expression in characters.
    """
    ...

def get_name_value(_from: ida_idaapi.ea_t, name: str) -> uval_t:
    r"""Get value of the name. This function knows about: regular names, enums, special segments, etc. 
            
    :param name: any name in the program or nullptr
    :returns: Name value result codes
    """
    ...

def get_nice_colored_name(ea: ida_idaapi.ea_t, flags: int = 0) -> str:
    r"""Get a nice colored name at the specified address. Ex:
    * segment:sub+offset
    * segment:sub:local_label
    * segment:label
    * segment:address
    * segment:address+offset
    
    
    
    :param ea: linear address
    :param flags: Nice colored name flags
    :returns: the length of the generated name in bytes.
    """
    ...

def get_nlist_ea(idx: size_t) -> ida_idaapi.ea_t:
    ...

def get_nlist_idx(ea: ida_idaapi.ea_t) -> int:
    ...

def get_nlist_name(idx: size_t) -> str:
    ...

def get_nlist_size() -> int:
    ...

def get_short_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    ...

def get_visible_name(ea: ida_idaapi.ea_t, gtn_flags: int = 0) -> str:
    ...

def hide_name(ea: ida_idaapi.ea_t) -> None:
    r"""Remove name from the list of names 
            
    :param ea: address of the name
    """
    ...

def is_ident(name: str) -> bool:
    r"""Is a valid name? (including ::MangleChars)
    
    """
    ...

def is_ident_cp(cp: wchar32_t) -> bool:
    r"""Can a character appear in a name? (present in ::NameChars or ::MangleChars)
    
    """
    ...

def is_in_nlist(ea: ida_idaapi.ea_t) -> bool:
    ...

def is_name_defined_locally(args: Any) -> bool:
    r"""Is the name defined locally in the specified function? 
            
    :param pfn: pointer to function
    :param name: name to check
    :param ignore_name_def: which names to ignore when checking
    :param ea1: the starting address of the range inside the function (optional)
    :param ea2: the ending address of the range inside the function (optional)
    :returns: true if the name has been defined
    """
    ...

def is_public_name(ea: ida_idaapi.ea_t) -> bool:
    ...

def is_strlit_cp(cp: wchar32_t, specific_ranges: rangeset_crefvec_t = None) -> bool:
    r"""Can a character appear in a string literal (present in ::StrlitChars) If 'specific_ranges' are specified, those will be used instead of the ones corresponding to the current culture (only if ::StrlitChars is configured to use the current culture) 
            
    """
    ...

def is_uname(name: str) -> bool:
    r"""Is valid user-specified name? (valid name & !dummy prefix). 
            
    :param name: name to test. may be nullptr.
    :returns: 1: yes
    :returns: 0: no
    """
    ...

def is_valid_cp(cp: wchar32_t, kind: nametype_t, data: void = None) -> bool:
    r"""Is the given codepoint acceptable in the given context?
    
    """
    ...

def is_valid_typename(name: str) -> bool:
    r"""Is valid type name? 
            
    :param name: name to test. may be nullptr.
    :returns: 1: yes
    :returns: 0: no
    """
    ...

def is_visible_cp(cp: wchar32_t) -> bool:
    r"""Can a character be displayed in a name? (present in ::NameChars)
    
    """
    ...

def is_weak_name(ea: ida_idaapi.ea_t) -> bool:
    ...

def make_name_auto(ea: ida_idaapi.ea_t) -> bool:
    ...

def make_name_non_public(ea: ida_idaapi.ea_t) -> None:
    ...

def make_name_non_weak(ea: ida_idaapi.ea_t) -> None:
    ...

def make_name_public(ea: ida_idaapi.ea_t) -> None:
    ...

def make_name_user(ea: ida_idaapi.ea_t) -> bool:
    ...

def make_name_weak(ea: ida_idaapi.ea_t) -> None:
    ...

def rebuild_nlist() -> None:
    ...

def reorder_dummy_names() -> None:
    r"""Renumber dummy names.
    
    """
    ...

def set_cp_validity(args: Any) -> None:
    r"""Mark the given codepoint (or range) as acceptable or unacceptable in the given context If 'endcp' is not BADCP, it is considered to be the end of the range: [cp, endcp), and is not included in the range 
            
    """
    ...

def set_debug_name(ea: ida_idaapi.ea_t, name: str) -> bool:
    ...

def set_dummy_name(_from: ida_idaapi.ea_t, ea: ida_idaapi.ea_t) -> bool:
    r"""Give an autogenerated (dummy) name. Autogenerated names have special prefixes (loc_...). 
            
    :param ea: linear address
    :returns: 1: ok, dummy name is generated or the byte already had a name
    :returns: 0: failure, invalid address or tail byte
    """
    ...

def set_name(ea: ida_idaapi.ea_t, name: str, flags: int = 0) -> bool:
    r"""Set or delete name of an item at the specified address. An item can be anything: instruction, function, data byte, word, string, structure, etc... Include name into the list of names. 
            
    :param ea: linear address. do nothing if ea is not valid (return 0). tail bytes can't have names.
    :param name: new name.
    * nullptr: do nothing (return 0).
    * "" : delete name.
    * otherwise this is a new name.
    :param flags: Set name flags. If a bit is not specified, then the corresponding action is not performed and the name will retain the same bits as before calling this function. For new names, default is: non-public, non-weak, non-auto.
    :returns: 1: ok, name is changed
    :returns: 0: failure, a warning is displayed
    """
    ...

def show_name(ea: ida_idaapi.ea_t) -> None:
    r"""Insert name to the list of names.
    
    """
    ...

def validate_name(name: str, type: nametype_t, flags: int = 1) -> Any:
    r"""Validate a name. If SN_NOCHECK is specified, this function replaces all invalid characters in the name with SUBSTCHAR. However, it will return false if name is valid but not allowed to be an identifier (is a register name).
    
    :param name: ptr to name. the name will be modified
    :param type: the type of name we want to validate
    :param flags: see SN_*
    :returns: success
    """
    ...

CN_KEEP_TRAILING_DIGITS: int  # 1
CN_KEEP_UNDERSCORES: int  # 2
DEBNAME_EXACT: int  # 0
DEBNAME_LOWER: int  # 1
DEBNAME_NICE: int  # 3
DEBNAME_UPPER: int  # 2
DQT_COMPILER: int  # 0
DQT_FULL: int  # 2
DQT_NAME_TYPE: int  # 1
DQT_NPURGED_2: int  # -2
DQT_NPURGED_4: int  # -4
DQT_NPURGED_8: int  # -8
FUNC_IMPORT_PREFIX: str  # __imp_
GETN_APPZERO: int  # 1
GETN_NODUMMY: int  # 4
GETN_NOFIXUP: int  # 2
GNCN_NOCOLOR: int  # 2
GNCN_NODBGNM: int  # 256
GNCN_NOFUNC: int  # 8
GNCN_NOLABEL: int  # 4
GNCN_NOSEG: int  # 1
GNCN_PREFDBG: int  # 512
GNCN_REQFUNC: int  # 64
GNCN_REQNAME: int  # 128
GNCN_SEGNUM: int  # 32
GNCN_SEG_FUNC: int  # 16
GN_COLORED: int  # 2
GN_DEMANGLED: int  # 4
GN_ISRET: int  # 128
GN_LOCAL: int  # 64
GN_LONG: int  # 32
GN_NOT_DUMMY: int  # 512
GN_NOT_ISRET: int  # 256
GN_SHORT: int  # 16
GN_STRICT: int  # 8
GN_VISIBLE: int  # 1
MANGLED_CODE: int  # 0
MANGLED_DATA: int  # 1
MANGLED_UNKNOWN: int  # 2
MAXNAMELEN: int  # 512
ME_ERRAUTO: int  # -7
ME_FRAME: int  # -5
ME_ILLSTR: int  # -3
ME_INTERR: int  # -1
ME_NOCOMP: int  # -6
ME_NOERROR_LIMIT: int  # -10
ME_NOHASHMEM: int  # -8
ME_NOSTRMEM: int  # -9
ME_PARAMERR: int  # -2
ME_SMALLANS: int  # -4
MNG_CALC_VALID: int  # 1979711488
MNG_COMPILER_MSK: int  # 1879048192
MNG_DEFFAR: int  # 2
MNG_DEFHUGE: int  # 4
MNG_DEFNEAR: int  # 0
MNG_DEFNEARANY: int  # 1
MNG_DEFNONE: int  # 6
MNG_DEFPTR64: int  # 5
MNG_DROP_IMP: int  # 8388608
MNG_IGN_ANYWAY: int  # 33554432
MNG_IGN_JMP: int  # 67108864
MNG_LONG_FORM: int  # 104857607
MNG_MOVE_JMP: int  # 134217728
MNG_NOBASEDT: int  # 128
MNG_NOCALLC: int  # 256
MNG_NOCLOSUR: int  # 32768
MNG_NOCSVOL: int  # 16384
MNG_NODEFINIT: int  # 8
MNG_NOECSU: int  # 8192
MNG_NOMANAGE: int  # 131072
MNG_NOMODULE: int  # 262144
MNG_NOPOSTFC: int  # 512
MNG_NOPTRTYP: int  # 7
MNG_NOPTRTYP16: int  # 3
MNG_NORETTYPE: int  # 64
MNG_NOSCTYP: int  # 1024
MNG_NOSTVIR: int  # 4096
MNG_NOTHROW: int  # 2048
MNG_NOTYPE: int  # 32
MNG_NOUNALG: int  # 65536
MNG_NOUNDERSCORE: int  # 16
MNG_PTRMSK: int  # 7
MNG_SHORT_FORM: int  # 245612135
MNG_SHORT_S: int  # 1048576
MNG_SHORT_U: int  # 2097152
MNG_ZPT_SPACE: int  # 4194304
MT_BORLAN: int  # 536870912
MT_CASTING: int  # 4194304
MT_CDECL: int  # 2
MT_CLRCALL: int  # 11
MT_CLRCDTOR: int  # 5242880
MT_CONSTR: int  # 2097152
MT_DEFAULT: int  # 1
MT_DESTR: int  # 3145728
MT_DMDCALL: int  # 12
MT_FASTCALL: int  # 5
MT_FORTRAN: int  # 7
MT_GCC3: int  # 1610612736
MT_GNU: int  # 1342177280
MT_INTERRUPT: int  # 9
MT_LOCALNAME: int  # 15
MT_MEMBER: int  # 128
MT_MSCOMP: int  # 268435456
MT_MSFASTCALL: int  # 10
MT_OPERAT: int  # 1048576
MT_OTHER: int  # 1073741824
MT_PARMAX: int  # 255
MT_PARSHF: int  # 8
MT_PASCAL: int  # 3
MT_PRIVATE: int  # 64
MT_PROTECT: int  # 96
MT_PUBLIC: int  # 32
MT_REGCALL: int  # 14
MT_RTTI: int  # 192
MT_STDCALL: int  # 4
MT_SYSCALL: int  # 8
MT_THISCALL: int  # 6
MT_VECTORCALL: int  # 13
MT_VISAGE: int  # 1879048192
MT_VOIDARG: int  # 130816
MT_VTABLE: int  # 160
MT_WATCOM: int  # 805306368
M_ANONNSP: int  # 33554432
M_AUTOCRT: int  # 524288
M_CLASS: int  # 224
M_COMPILER: int  # 1879048192
M_DBGNAME: int  # 134217728
M_ELLIPSIS: int  # 65536
M_PARMSK: int  # 65280
M_PRCMSK: int  # 15
M_SAVEREGS: int  # 16
M_STATIC: int  # 131072
M_THUNK: int  # 16777216
M_TMPLNAM: int  # 67108864
M_TRUNCATE: int  # 8388608
M_TYPMASK: int  # 7340032
M_VIRTUAL: int  # 262144
NT_ABS: int  # 5
NT_BMASK: int  # 8
NT_BYTE: int  # 1
NT_ENUM: int  # 4
NT_LOCAL: int  # 2
NT_NONE: int  # 0
NT_REGVAR: int  # 9
NT_SEG: int  # 6
NT_STKVAR: int  # 3
NT_STROFF: int  # 7
SN_AUTO: int  # 32
SN_CHECK: int  # 0
SN_DELTAIL: int  # 8192
SN_FORCE: int  # 2048
SN_IDBENC: int  # 1024
SN_LOCAL: int  # 512
SN_MULTI: int  # 16384
SN_MULTI_FORCE: int  # 32768
SN_NOCHECK: int  # 1
SN_NODUMMY: int  # 4096
SN_NOLIST: int  # 128
SN_NON_AUTO: int  # 64
SN_NON_PUBLIC: int  # 4
SN_NON_WEAK: int  # 16
SN_NOWARN: int  # 256
SN_PUBLIC: int  # 2
SN_WEAK: int  # 8
SWIG_PYTHON_LEGACY_BOOL: int  # 1
UCDR_MANGLED: int  # 4
UCDR_NAME: int  # 2
UCDR_STRLIT: int  # 1
UCDR_TYPE: int  # 8
VNT_IDENT: int  # 6
VNT_STRLIT: int  # 1
VNT_TYPE: int  # 8
VNT_UDTMEM: int  # 2
VNT_VISIBLE: int  # 2
annotations: _Feature  # _Feature((3, 7, 0, 'beta', 1), None, 16777216)
bisect: module
cvar: swigvarlink
ida_idaapi: module
ignore_glabel: int  # 4
ignore_llabel: int  # 2
ignore_none: int  # 0
ignore_regvar: int  # 1
ignore_stkvar: int  # 3
weakref: module


from typing import Any, Optional, List, Dict, Tuple, Callable, Union

r"""Functions that deal with C-like expressions and built-in IDC language.

Functions marked THREAD_SAFE may be called from any thread. No simultaneous calls should be made for the same variable. We protect only global structures, individual variables must be protected manually. 
    
"""

class highlighter_cbs_t:
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __disown__(self) -> Any:
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def cur_block_state(self) -> int:
        ...
    def prev_block_state(self) -> int:
        ...
    def set_block_state(self, arg0: int) -> None:
        ...
    def set_style(self, arg0: int, arg1: int, arg2: syntax_highlight_style) -> None:
        ...

class idc_global_t:
    @property
    def name(self) -> Any: ...
    @property
    def value(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...

class idc_value_t:
    @property
    def e(self) -> Any: ...
    @property
    def funcidx(self) -> Any: ...
    @property
    def i64(self) -> Any: ...
    @property
    def num(self) -> Any: ...
    @property
    def obj(self) -> Any: ...
    @property
    def pvoid(self) -> Any: ...
    @property
    def reserve(self) -> Any: ...
    @property
    def str(self) -> Any: ...
    @property
    def vtype(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def c_str(self) -> str:
        r"""VT_STR
        
        """
        ...
    def clear(self) -> None:
        r"""See free_idcv()
        
        """
        ...
    def create_empty_string(self) -> None:
        ...
    def is_convertible(self) -> bool:
        r"""Convertible types are VT_LONG, VT_FLOAT, VT_INT64, and VT_STR.
        
        """
        ...
    def is_integral(self) -> bool:
        r"""Does value represent a whole number? 
                
        """
        ...
    def is_zero(self) -> bool:
        r"""Does value represent the integer 0?
        
        """
        ...
    def qstr(self) -> str:
        r"""VT_STR
        
        """
        ...
    def set_float(self, f: fpvalue_t) -> None:
        ...
    def set_int64(self, v: int64) -> None:
        ...
    def set_long(self, v: int) -> None:
        ...
    def set_pvoid(self, p: void) -> None:
        ...
    def set_string(self, args: Any) -> None:
        ...
    def swap(self, v: idc_value_t) -> None:
        r"""Set this = r and v = this.
        
        """
        ...
    def u_str(self) -> uchar:
        r"""VT_STR
        
        """
        ...

class idc_values_t:
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getitem__(self, i: size_t) -> idc_value_t:
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __iter__(self) -> Any:
        r"""Helper function, to be set as __iter__ method for qvector-, or array-based classes."""
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __len__(self) -> int:
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __setitem__(self, i: size_t, v: idc_value_t) -> None:
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def append(self, x: idc_value_t) -> None:
        ...
    def at(self, _idx: size_t) -> idc_value_t:
        ...
    def back(self) -> Any:
        ...
    def begin(self, args: Any) -> const_iterator:
        ...
    def capacity(self) -> int:
        ...
    def clear(self) -> None:
        ...
    def empty(self) -> bool:
        ...
    def end(self, args: Any) -> const_iterator:
        ...
    def erase(self, args: Any) -> iterator:
        ...
    def extend(self, x: idc_values_t) -> None:
        ...
    def extract(self) -> idc_value_t:
        ...
    def front(self) -> Any:
        ...
    def grow(self, args: Any) -> None:
        ...
    def inject(self, s: idc_value_t, len: size_t) -> None:
        ...
    def insert(self, it: idc_value_t, x: idc_value_t) -> iterator:
        ...
    def pop_back(self) -> None:
        ...
    def push_back(self, args: Any) -> idc_value_t:
        ...
    def qclear(self) -> None:
        ...
    def reserve(self, cnt: size_t) -> None:
        ...
    def resize(self, args: Any) -> None:
        ...
    def size(self) -> int:
        ...
    def swap(self, r: idc_values_t) -> None:
        ...
    def truncate(self) -> None:
        ...

def add_idc_class(name: str, super: idc_class_t = None) -> idc_class_t:
    r"""Create a new IDC class. 
            
    :param name: name of the new class
    :param super: the base class for the new class. if the new class is not based on any other class, pass nullptr
    :returns: pointer to the created class. If such a class already exists, a pointer to it will be returned. Pointers to other existing classes may be invalidated by this call.
    """
    ...

def add_idc_func(name: Any, fp: Any, args: Any, defvals: Any = ..., flags: Any = 0) -> Any:
    r"""Add an IDC function. This function does not modify the predefined kernel functions. Example: 
         error_t idaapi myfunc5(idc_value_t *argv, idc_value_t *res)
        
          msg("myfunc is called with arg0=%a and arg1=%s\n", argv[0].num, argv[1].str);
          res->num = 5;     // let's return 5
          return eOk;
        
         const char myfunc5_args[] = { VT_LONG, VT_STR, 0 };
         const ext_idcfunc_t myfunc_desc = { "MyFunc5", myfunc5, myfunc5_args, nullptr, 0, EXTFUN_BASE };
        
         after this:
        
        
         there is a new IDC function which can be called like this:
         "test");
    
    
            
    :returns: success
    """
    ...

def add_idc_gvar(name: str) -> idc_value_t:
    r"""Add global IDC variable. 
            
    :param name: name of the global variable
    :returns: pointer to the created variable or existing variable. NB: the returned pointer is valid until a new global var is added.
    """
    ...

def call_idc_func__(args: Any, kwargs: Any) -> Any:
    ...

def compile_idc_file(nonnul_line: str) -> str:
    ...

def compile_idc_snippet(func: str, text: str, resolver: idc_resolver_t = None, only_safe_funcs: bool = False) -> str:
    r"""Compile text with IDC statements. 
            
    :param func: name of the function to create out of the snippet
    :param text: text to compile
    :param resolver: callback object to get values of undefined variables This object will be called if IDC function contains references to undefined variables. May be nullptr.
    :param only_safe_funcs: if true, any calls to functions without EXTFUN_SAFE flag will lead to a compilation error.
    :returns: true: ok
    :returns: false: error, see errbuf
    """
    ...

def compile_idc_text(nonnul_line: str) -> str:
    ...

def copy_idcv(dst: idc_value_t, src: idc_value_t) -> error_t:
    r"""Copy 'src' to 'dst'. For idc objects only a reference is copied. 
            
    """
    ...

def create_idcv_ref(ref: idc_value_t, v: idc_value_t) -> bool:
    r"""Create a variable reference. Currently only references to global variables can be created. 
            
    :param ref: ptr to the result
    :param v: variable to reference
    :returns: success
    """
    ...

def deep_copy_idcv(dst: idc_value_t, src: idc_value_t) -> error_t:
    r"""Deep copy an IDC object. This function performs deep copy of idc objects. If 'src' is not an object, copy_idcv() will be called 
            
    """
    ...

def del_idc_func(name: Any) -> Any:
    r"""Delete an IDC function 
            
    """
    ...

def del_idcv_attr(obj: idc_value_t, attr: str) -> error_t:
    r"""Delete an object attribute. 
            
    :param obj: variable that holds an object reference
    :param attr: attribute name
    :returns: error code, eOk on success
    """
    ...

def deref_idcv(v: idc_value_t, vref_flags: int) -> idc_value_t:
    r"""Dereference a VT_REF variable. 
            
    :param v: variable to dereference
    :param vref_flags: Dereference IDC variable flags
    :returns: pointer to the dereference result or nullptr. If returns nullptr, qerrno is set to eExecBadRef "Illegal variable reference"
    """
    ...

def eval_expr(rv: idc_value_t, where: ida_idaapi.ea_t, line: str) -> str:
    r"""Compile and calculate an expression. 
            
    :param rv: pointer to the result
    :param where: the current linear address in the addressing space of the program being disassembled. If will be used to resolve names of local variables etc. if not applicable, then should be BADADDR.
    :param line: the expression to evaluate
    :returns: true: ok
    :returns: false: error, see errbuf
    """
    ...

def eval_idc_expr(rv: idc_value_t, where: ida_idaapi.ea_t, line: str) -> str:
    r"""Same as eval_expr(), but will always use the IDC interpreter regardless of the currently installed extlang. 
            
    """
    ...

def exec_idc_script(result: idc_value_t, path: str, func: str, args: idc_value_t, argsnum: size_t) -> str:
    r"""Compile and execute IDC function(s) from file. 
            
    :param result: ptr to idc_value_t to hold result of the function. If execution fails, this variable will contain the exception information. You may pass nullptr if you are not interested in the returned value.
    :param path: text file containing text of IDC functions
    :param func: function name to execute
    :param args: array of parameters
    :param argsnum: number of parameters to pass to 'fname' This number should be equal to number of parameters the function expects.
    :returns: true: ok
    :returns: false: error, see errbuf
    """
    ...

def exec_system_script(file: str, complain_if_no_file: bool = True) -> bool:
    r"""Compile and execute "main" function from system file. 
            
    :param file: file name with IDC function(s). The file will be searched using get_idc_filename().
    :param complain_if_no_file: * 1: display warning if the file is not found
    * 0: don't complain if file doesn't exist
    :returns: 1: ok, file is compiled and executed
    :returns: 0: failure, compilation or execution error, warning is displayed
    """
    ...

def find_idc_class(name: str) -> idc_class_t:
    r"""Find an existing IDC class by its name. 
            
    :param name: name of the class
    :returns: pointer to the class or nullptr. The returned pointer is valid until a new call to add_idc_class()
    """
    ...

def find_idc_func(prefix: str, n: int = 0) -> str:
    ...

def find_idc_gvar(name: str) -> idc_value_t:
    r"""Find an existing global IDC variable by its name. 
            
    :param name: name of the global variable
    :returns: pointer to the variable or nullptr. NB: the returned pointer is valid until a new global var is added. FIXME: it is difficult to use this function in a thread safe manner
    """
    ...

def first_idcv_attr(obj: idc_value_t) -> str:
    ...

def free_idcv(v: idc_value_t) -> None:
    r"""Free storage used by VT_STR/VT_OBJ IDC variables. After this call the variable has a numeric value 0 
            
    """
    ...

def get_idc_filename(file: str) -> str:
    r"""Get full name of IDC file name. Search for file in list of include directories, IDCPATH directory and system directories. 
            
    :param file: file name without full path
    :returns: nullptr is file not found. otherwise returns pointer to buf
    """
    ...

def get_idcv_attr(res: idc_value_t, obj: idc_value_t, attr: str, may_use_getattr: bool = False) -> error_t:
    r"""Get an object attribute. 
            
    :param res: buffer for the attribute value
    :param obj: variable that holds an object reference. if obj is nullptr it searches global variables, then user functions
    :param attr: attribute name
    :param may_use_getattr: may call getattr functions to calculate the attribute if it does not exist
    :returns: error code, eOk on success
    """
    ...

def get_idcv_class_name(obj: idc_value_t) -> str:
    r"""Retrieves the IDC object class name. 
            
    :param obj: class instance variable
    :returns: error code, eOk on success
    """
    ...

def get_idcv_slice(res: idc_value_t, v: idc_value_t, i1: int, i2: int, flags: int = 0) -> error_t:
    r"""Get slice. 
            
    :param res: output variable that will contain the slice
    :param v: input variable (string or object)
    :param i1: slice start index
    :param i2: slice end index (excluded)
    :param flags: IDC variable slice flags or 0
    :returns: eOk if success
    """
    ...

def idcv_float(v: idc_value_t) -> error_t:
    r"""Convert IDC variable to a floating point.
    
    """
    ...

def idcv_int64(v: idc_value_t) -> error_t:
    r"""Convert IDC variable to a 64bit number. 
            
    :returns: v = 0 if impossible to convert to int64
    """
    ...

def idcv_long(v: idc_value_t) -> error_t:
    r"""Convert IDC variable to a long (32/64bit) number. 
            
    :returns: v = 0 if impossible to convert to long
    """
    ...

def idcv_num(v: idc_value_t) -> error_t:
    r"""Convert IDC variable to a long number. 
            
    :returns: * v = 0 if IDC variable = "false" string
    * v = 1 if IDC variable = "true" string
    * v = number if IDC variable is number or string containing a number
    * eTypeConflict if IDC variable = empty string
    """
    ...

def idcv_object(v: idc_value_t, icls: idc_class_t = None) -> error_t:
    r"""Create an IDC object. The original value of 'v' is discarded (freed). 
            
    :param v: variable to hold the object. any previous value will be cleaned
    :param icls: ptr to the desired class. nullptr means "object" class this ptr must be returned by add_idc_class() or find_idc_class()
    :returns: always eOk
    """
    ...

def idcv_string(v: idc_value_t) -> error_t:
    r"""Convert IDC variable to a text string.
    
    """
    ...

def last_idcv_attr(obj: idc_value_t) -> str:
    ...

def move_idcv(dst: idc_value_t, src: idc_value_t) -> error_t:
    r"""Move 'src' to 'dst'. This function is more effective than copy_idcv since it never copies big amounts of data. 
            
    """
    ...

def next_idcv_attr(obj: idc_value_t, attr: str) -> str:
    ...

def prev_idcv_attr(obj: idc_value_t, attr: str) -> str:
    ...

def print_idcv(v: idc_value_t, name: str = None, indent: int = 0) -> str:
    r"""Get text representation of idc_value_t.
    
    """
    ...

def py_add_idc_func(name: str, fp_ptr: size_t, args: str, defvals: idc_values_t, flags: int) -> bool:
    ...

def py_get_call_idc_func() -> int:
    ...

def pyw_convert_defvals(out: idc_values_t, py_seq: Any) -> bool:
    ...

def pyw_register_idc_func(name: str, args: str, py_fp: Any) -> int:
    ...

def pyw_unregister_idc_func(ctxptr: size_t) -> bool:
    ...

def set_header_path(path: str, add: bool) -> bool:
    r"""Set or append a header path. IDA looks for the include files in the appended header paths, then in the ida executable directory. 
            
    :param path: list of directories to add (separated by ';') may be nullptr, in this case nothing is added
    :param add: true: append. false: remove old paths.
    :returns: true: success
    :returns: false: no memory
    """
    ...

def set_idcv_attr(obj: idc_value_t, attr: str, value: idc_value_t, may_use_setattr: bool = False) -> error_t:
    r"""Set an object attribute. 
            
    :param obj: variable that holds an object reference. if obj is nullptr then it tries to modify a global variable with the attribute name
    :param attr: attribute name
    :param value: new attribute value
    :param may_use_setattr: may call setattr functions for the class
    :returns: error code, eOk on success
    """
    ...

def set_idcv_slice(v: idc_value_t, i1: int, i2: int, _in: idc_value_t, flags: int = 0) -> error_t:
    r"""Set slice. 
            
    :param v: variable to modify (string or object)
    :param i1: slice start index
    :param i2: slice end index (excluded)
    :param flags: IDC variable slice flags or 0
    :returns: eOk on success
    """
    ...

def swap_idcvs(v1: idc_value_t, v2: idc_value_t) -> None:
    r"""Swap 2 variables.
    
    """
    ...

def throw_idc_exception(r: idc_value_t, desc: str) -> error_t:
    r"""Create an idc execution exception object. This helper function can be used to return an exception from C++ code to IDC. In other words this function can be called from idc_func_t() callbacks. Sample usage: if ( !ok ) return throw_idc_exception(r, "detailed error msg"); 
            
    :param r: object to hold the exception object
    :param desc: exception description
    :returns: eExecThrow
    """
    ...

CPL_DEL_MACROS: int  # 1
CPL_ONLY_SAFE: int  # 4
CPL_USE_LABELS: int  # 2
EXTFUN_BASE: int  # 1
EXTFUN_NORET: int  # 2
EXTFUN_SAFE: int  # 4
HF_COMMENT: int  # 5
HF_DEFAULT: int  # 0
HF_KEYWORD1: int  # 1
HF_KEYWORD2: int  # 2
HF_KEYWORD3: int  # 3
HF_MAX: int  # 12
HF_NUMBER: int  # 7
HF_PREPROC: int  # 6
HF_STRING: int  # 4
HF_USER1: int  # 8
HF_USER2: int  # 9
HF_USER3: int  # 10
HF_USER4: int  # 11
IDC_LANG_EXT: str  # idc
SWIG_PYTHON_LEGACY_BOOL: int  # 1
VARSLICE_SINGLE: int  # 1
VREF_COPY: int  # 2
VREF_LOOP: int  # 0
VREF_ONCE: int  # 1
VT_FLOAT: int  # 3
VT_FUNC: int  # 6
VT_INT64: int  # 9
VT_LONG: int  # 2
VT_OBJ: int  # 5
VT_PVOID: int  # 8
VT_REF: int  # 10
VT_STR: int  # 7
VT_WILD: int  # 4
annotations: _Feature  # _Feature((3, 7, 0, 'beta', 1), None, 16777216)
ctypes: module
eExecThrow: int  # 90
ida_idaapi: module
types: module
weakref: module


from typing import Any, Optional, List, Dict, Tuple, Callable, Union

r"""Functions that deal with the string list.

While the kernel keeps the string list, it does not update it. The string list is not used by the kernel because keeping it up-to-date would slow down IDA without any benefit. If the string list is not cleared using clear_strlist(), the list will be saved to the database and restored on the next startup.
The users of this list should call build_strlist() if they need an up-to-date version. 
    
"""

class string_info_t:
    @property
    def ea(self) -> Any: ...
    @property
    def length(self) -> Any: ...
    @property
    def type(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, r: string_info_t) -> bool:
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...

class strwinsetup_t:
    @property
    def display_only_existing_strings(self) -> Any: ...
    @property
    def ignore_heads(self) -> Any: ...
    @property
    def minlen(self) -> Any: ...
    @property
    def only_7bit(self) -> Any: ...
    @property
    def strtypes(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...

def build_strlist() -> None:
    r"""Rebuild the string list.
    
    """
    ...

def clear_strlist() -> None:
    r"""Clear the string list.
    
    """
    ...

def get_strlist_item(si: string_info_t, n: size_t) -> bool:
    r"""Get nth element of the string list (n=0..get_strlist_qty()-1)
    
    """
    ...

def get_strlist_options() -> strwinsetup_t:
    r"""Get the static string list options.
    
    """
    ...

def get_strlist_qty() -> int:
    r"""Get number of elements in the string list. The list will be loaded from the database (if saved) or built from scratch. 
            
    """
    ...

SWIG_PYTHON_LEGACY_BOOL: int  # 1
annotations: _Feature  # _Feature((3, 7, 0, 'beta', 1), None, 16777216)
ida_idaapi: module
weakref: module


from __future__ import annotations
import sys
import platform
from pathlib import Path
from typing import Optional
import ctypes
import os
from .config import get_ida_install_dir

def find_file(name, path):
    """Internal method used for recursively find a file in a folder"""
    for root, _, files in os.walk(path):
        if name in files:
            return os.path.join(root, name)
    return None

# get the right filename based on platform
platform_str = platform.system()

if platform_str == "Windows":
    name = "idalib.dll"
elif platform_str == "Linux":
    name = "libidalib.so"
elif platform_str == "Darwin":
    name = "libidalib.dylib"
else:
    raise ImportError(f"Unknown platform {platform_str}")

# Get the ida-install-dir setting
root_dir = get_ida_install_dir()
idalib_path = find_file(name=name, path=root_dir) if root_dir else None
if idalib_path is None:
    raise ImportError(f"Cannot load IDA library file {name}. Please make sure you are using IDA version 9.0 or newer and the IDADIR environment variable is set and points to the active IDA installation.")

if "IDA_IS_INTERACTIVE" in os.environ:
    if os.environ["IDA_IS_INTERACTIVE"] == "1":
        raise ImportError("The IDA library can only be run in a separate process and cannot be loaded within IDA itself.")

# load the library and initialize the kernel
try:
    if platform_str == "Windows":
        # IDA kernel and modules may need dependencies in its root dir (e.g. clp64.dll)
        # tell Windows loader where they can be found
        # NB: for AddDllDirectory() be effective for indirectly loaded DLLs,
        # SetDefaultDllDirectories(LOAD_LIBRARY_SEARCH_USER_DIRS) has to be called first
        kernel32 = ctypes.WinDLL('kernel32', use_last_error=True)
        SetDefaultDllDirectories = getattr(kernel32, "SetDefaultDllDirectories")
        if SetDefaultDllDirectories:
            from  ctypes.wintypes import DWORD, BOOL
            SetDefaultDllDirectories.argtypes = [DWORD]
            SetDefaultDllDirectories.restype = BOOL
            LOAD_LIBRARY_SEARCH_DEFAULT_DIRS = 0x00001000
            LOAD_LIBRARY_SEARCH_USER_DIRS = 0x00000400
            flags = LOAD_LIBRARY_SEARCH_DEFAULT_DIRS | LOAD_LIBRARY_SEARCH_USER_DIRS
            if SetDefaultDllDirectories(flags):
                # finally, add IDA's directory for dependencies
                os.add_dll_directory(root_dir)
        else:
            os.environ['PATH'] = root_dir + os.pathsep + os.environ.get('PATH', '')

    libida = ctypes.cdll.LoadLibrary(idalib_path)
except Exception as e:
    raise ImportError(f"Failed loading IDA library file {idalib_path}, exception {e}\n")

try:
    error_description = None
    init_code = libida.init_library(0, None)
    if init_code != 0:
        error_description = f"init_library error code {init_code}"
except Exception as e:
    error_description = f"exception {e}"

if error_description != None:
    raise ImportError(f"Failed to initialize IDA library, {error_description}, check logging for additional information\n")

sys.path.append(str(Path(idalib_path).parent / "python/lib-dynload"))
sys.path.append(str(Path(idalib_path).parent / "python"))

def open_database(file_name:str, run_auto_analysis: bool, args: Optional[str] = None, enable_history: bool = False)->int:
    """Opens the database specified by the file_name argument.
    If run_auto_analysis is set to True, the auto-analysis will start and wait for its completion.
    If enable_history is set to True, update the file history list; the default within idalib is not to update it.
    Optionally, additional arguments can be provided via args parameter, which follow the same format as IDA's command-line arguments.
    NOTE: All library functions must be called from the same thread that initialized the library.
    The library is single-threaded, and performing database operations from a different thread may result in undefined behavior."""
    if enable_history:
        args = (args or "") + " -Oidalib:enable_history"
    return libida.open_database(file_name.encode(), run_auto_analysis, args.encode() if args is not None else None)

def close_database(save = True)->None:
    """Close the current database"""
    libida.close_database(save)

def make_signatures(only_pat = False)->bool:
    """Generate .sig and .pat files for the current database"""
    return libida.make_signatures(only_pat)

def enable_console_messages(enable:bool)->None:
    """Enable console messages, they are disabled by default"""
    libida.enable_console_messages(enable)

def set_screen_ea(screen_ea: "ea_t")->None:
    """Set screen ea, let the user specify the current screen ea
    subsequent calls to get_screen_ea will return this value"""
    libida.set_screen_ea.argtypes = [ctypes.c_uint64]
    libida.set_screen_ea(screen_ea)

def get_library_version()->tuple[int, int, int] | None:
    """Get IDA library version, return minor, major build in case of success, None if fails"""
    major = ctypes.c_int()
    minor = ctypes.c_int()
    build = ctypes.c_int()

    libida.get_library_version.argtypes = [ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int)]
    libida.get_library_version.restype = ctypes.c_bool
    success = libida.get_library_version(ctypes.byref(major), ctypes.byref(minor), ctypes.byref(build))

    if success:
        return major.value, minor.value, build.value
    else:
        return None



from typing import Any, Optional, List, Dict, Tuple, Callable, Union

class CustomIDAMemo:
    def CreateGroups(self, groups_infos: Any) -> Any:
        r"""
        Send a request to modify the graph by creating a
        (set of) group(s), and perform an animation.
        
        Each object in the 'groups_infos' list must be of the format:
        {
          "nodes" : [<int>, <int>, <int>, ...] # The list of nodes to group
          "text" : <string>                    # The synthetic text for that group
        }
        
        :param groups_infos: A list of objects that describe those groups.
        :returns: A [<int>, <int>, ...] list of group nodes, or None (failure).
        
        """
        ...
    def DelNodesInfos(self, nodes: Any) -> Any:
        r"""
        Delete the properties for the given node(s).
        
        :param nodes: A list of node IDs
        
        """
        ...
    def DeleteGroups(self, groups: Any, new_current: Any = -1) -> Any:
        r"""
        Send a request to delete the specified groups in the graph,
        and perform an animation.
        
        :param groups: A list of group node numbers.
        :param new_current: A node to focus on after the groups have been deleted
        :returns: True on success, False otherwise.
        
        """
        ...
    def GetCurrentRendererType(self) -> Any:
        ...
    def GetNodeInfo(self, args: Any) -> Any:
        r"""
        Get the properties for the given node.
        
        :param ni: A node_info_t instance
        :param node: The index of the node.
        :returns: success
        
        """
        ...
    def GetWidget(self) -> Any:
        r"""
        Return the TWidget underlying this view.
        
        :returns: The TWidget underlying this view, or None.
        
        """
        ...
    def GetWidgetAsGraphViewer(self) -> Any:
        r"""
        Return the graph_viewer_t underlying this view.
        
        :returns: The graph_viewer_t underlying this view, or None.
        
        """
        ...
    def Refresh(self) -> Any:
        r"""
        Refreshes the view. This causes the OnRefresh() to be called
        
        """
        ...
    def SetCurrentRendererType(self, rtype: Any) -> Any:
        r"""
        Set the current view's renderer.
        
        :param rtype: The renderer type. Should be one of the idaapi.TCCRT_* values.
        
        """
        ...
    def SetGroupsVisibility(self, groups: Any, expand: Any, new_current: Any = -1) -> Any:
        r"""
        Send a request to expand/collapse the specified groups in the graph,
        and perform an animation.
        
        :param groups: A list of group node numbers.
        :param expand: True to expand the group, False otherwise.
        :param new_current: A node to focus on after the groups have been expanded/collapsed.
        :returns: True on success, False otherwise.
        
        """
        ...
    def SetNodeInfo(self, node_index: Any, node_info: Any, flags: Any) -> Any:
        r"""
        Set the properties for the given node.
        
        Example usage (set second nodes's bg color to red):
          inst = ...
          p = idaapi.node_info_t()
          p.bg_color = 0x00ff0000
          inst.SetNodeInfo(1, p, idaapi.NIF_BG_COLOR)
        
        :param node_index: The node index.
        :param node_info: An idaapi.node_info_t instance.
        :param flags: An OR'ed value of NIF_* values.
        
        """
        ...
    def SetNodesInfos(self, values: Any) -> Any:
        r"""
        Set the properties for the given nodes.
        
        Example usage (set first three nodes's bg color to purple):
          inst = ...
          p = idaapi.node_info_t()
          p.bg_color = 0x00ff00ff
          inst.SetNodesInfos({0 : p, 1 : p, 2 : p})
        
        :param values: A dictionary of 'int -> node_info_t' objects.
        
        """
        ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __disown__(self) -> Any:
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def hook(self) -> bool:
        ...
    def unhook(self) -> bool:
        ...
    def view_activated(self, view: Any) -> Any:
        ...
    def view_click(self, view: Any, ve: Any) -> Any:
        ...
    def view_close(self, view: Any, args: Any) -> Any:
        ...
    def view_created(self, view: TWidget) -> None:
        r"""A view is being created. 
                  
        :param view: (TWidget *)
        """
        ...
    def view_curpos(self, view: Any, args: Any) -> Any:
        ...
    def view_dblclick(self, view: Any, ve: Any) -> Any:
        ...
    def view_deactivated(self, view: Any) -> Any:
        ...
    def view_keydown(self, view: Any, key: Any, state: Any) -> Any:
        ...
    def view_loc_changed(self, view: Any, now: Any, was: Any) -> Any:
        ...
    def view_mouse_moved(self, view: Any, ve: Any) -> Any:
        ...
    def view_mouse_over(self, view: Any, ve: Any) -> Any:
        ...
    def view_switched(self, view: Any, rt: Any) -> Any:
        ...

class IDAPython_displayhook:
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def displayhook(self, item: Any) -> Any:
        ...
    def displayhook_format(self, item: Any) -> Any:
        ...
    def format_item(self, num_printer: Any, storage: Any, item: Any) -> Any:
        ...
    def format_seq(self, num_printer: Any, storage: Any, item: Any, opn: Any, cls: Any) -> Any:
        ...

class PyIdc_cvt_int64__(pyidc_cvt_helper__):
    r"""Helper class for explicitly representing VT_INT64 values"""
    @property
    def value(self) -> Any: ...
    def __add__(self, other: Any) -> Any:
        ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __div__(self, other: Any) -> Any:
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, v: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __mul__(self, other: Any) -> Any:
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __radd__(self, other: Any) -> Any:
        ...
    def __rdiv__(self, other: Any) -> Any:
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __rmul__(self, other: Any) -> Any:
        ...
    def __rsub__(self, other: Any) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __sub__(self, other: Any) -> Any:
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...

class PyIdc_cvt_refclass__(pyidc_cvt_helper__):
    r"""Helper class for representing references to immutable objects"""
    @property
    def value(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, v: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def cstr(self) -> Any:
        r"""Returns the string as a C string (up to the zero termination)"""
        ...

class ea_t:
    r"""int([x]) -> integer
    int(x, base=10) -> integer
    
    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating-point
    numbers, this truncates towards zero.
    
    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """
    denominator: getset_descriptor  # <attribute 'denominator' of 'int' objects>
    imag: getset_descriptor  # <attribute 'imag' of 'int' objects>
    numerator: getset_descriptor  # <attribute 'numerator' of 'int' objects>
    real: getset_descriptor  # <attribute 'real' of 'int' objects>
    def __abs__(self) -> Any:
        r"""abs(self)"""
        ...
    def __add__(self, value: Any) -> Any:
        r"""Return self+value."""
        ...
    def __and__(self, value: Any) -> Any:
        r"""Return self&value."""
        ...
    def __bool__(self) -> Any:
        r"""True if self else False"""
        ...
    def __ceil__(self) -> Any:
        r"""Ceiling of an Integral returns itself."""
        ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __divmod__(self, value: Any) -> Any:
        r"""Return divmod(self, value)."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __float__(self) -> Any:
        r"""float(self)"""
        ...
    def __floor__(self) -> Any:
        r"""Flooring an Integral returns itself."""
        ...
    def __floordiv__(self, value: Any) -> Any:
        r"""Return self//value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Convert to a string according to format_spec."""
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getnewargs__(self) -> Any:
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __index__(self) -> Any:
        r"""Return self converted to an integer, if self is suitable for use as an index into a list."""
        ...
    def __init__(self, args: Any, kwargs: Any) -> Any:
        r"""Initialize self.  See help(type(self)) for accurate signature."""
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __int__(self) -> Any:
        r"""int(self)"""
        ...
    def __invert__(self) -> Any:
        r"""~self"""
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lshift__(self, value: Any) -> Any:
        r"""Return self<<value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __mod__(self, value: Any) -> Any:
        r"""Return self%value."""
        ...
    def __mul__(self, value: Any) -> Any:
        r"""Return self*value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __neg__(self) -> Any:
        r"""-self"""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __or__(self, value: Any) -> Any:
        r"""Return self|value."""
        ...
    def __pos__(self) -> Any:
        r"""+self"""
        ...
    def __pow__(self, value: Any, mod: Any = None) -> Any:
        r"""Return pow(self, value, mod)."""
        ...
    def __radd__(self, value: Any) -> Any:
        r"""Return value+self."""
        ...
    def __rand__(self, value: Any) -> Any:
        r"""Return value&self."""
        ...
    def __rdivmod__(self, value: Any) -> Any:
        r"""Return divmod(value, self)."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __rfloordiv__(self, value: Any) -> Any:
        r"""Return value//self."""
        ...
    def __rlshift__(self, value: Any) -> Any:
        r"""Return value<<self."""
        ...
    def __rmod__(self, value: Any) -> Any:
        r"""Return value%self."""
        ...
    def __rmul__(self, value: Any) -> Any:
        r"""Return value*self."""
        ...
    def __ror__(self, value: Any) -> Any:
        r"""Return value|self."""
        ...
    def __round__(self, *args: Any, **kwargs: Any) -> Any:
        r"""Rounding an Integral returns itself.
        
        Rounding with an ndigits argument also returns an integer.
        """
        ...
    def __rpow__(self, value: Any, mod: Any = None) -> Any:
        r"""Return pow(value, self, mod)."""
        ...
    def __rrshift__(self, value: Any) -> Any:
        r"""Return value>>self."""
        ...
    def __rshift__(self, value: Any) -> Any:
        r"""Return self>>value."""
        ...
    def __rsub__(self, value: Any) -> Any:
        r"""Return value-self."""
        ...
    def __rtruediv__(self, value: Any) -> Any:
        r"""Return value/self."""
        ...
    def __rxor__(self, value: Any) -> Any:
        r"""Return value^self."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Returns size in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __sub__(self, value: Any) -> Any:
        r"""Return self-value."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __truediv__(self, value: Any) -> Any:
        r"""Return self/value."""
        ...
    def __trunc__(self) -> Any:
        r"""Truncating an Integral returns itself."""
        ...
    def __xor__(self, value: Any) -> Any:
        r"""Return self^value."""
        ...
    def as_integer_ratio(self) -> Any:
        r"""Return a pair of integers, whose ratio is equal to the original int.
        
        The ratio is in lowest terms and has a positive denominator.
        
        >>> (10).as_integer_ratio()
        (10, 1)
        >>> (-10).as_integer_ratio()
        (-10, 1)
        >>> (0).as_integer_ratio()
        (0, 1)
        """
        ...
    def bit_count(self) -> Any:
        r"""Number of ones in the binary representation of the absolute value of self.
        
        Also known as the population count.
        
        >>> bin(13)
        '0b1101'
        >>> (13).bit_count()
        3
        """
        ...
    def bit_length(self) -> Any:
        r"""Number of bits necessary to represent self in binary.
        
        >>> bin(37)
        '0b100101'
        >>> (37).bit_length()
        6
        """
        ...
    def conjugate(self) -> Any:
        r"""Returns self, the complex conjugate of any int."""
        ...
    def from_bytes(self, bytes: Any, byteorder: Any = 'big', signed: Any = False) -> Any:
        r"""Return the integer represented by the given array of bytes.
        
          bytes
            Holds the array of bytes to convert.  The argument must either
            support the buffer protocol or be an iterable object producing bytes.
            Bytes and bytearray are examples of built-in objects that support the
            buffer protocol.
          byteorder
            The byte order used to represent the integer.  If byteorder is 'big',
            the most significant byte is at the beginning of the byte array.  If
            byteorder is 'little', the most significant byte is at the end of the
            byte array.  To request the native byte order of the host system, use
            sys.byteorder as the byte order value.  Default is to use 'big'.
          signed
            Indicates whether two's complement is used to represent the integer.
        """
        ...
    def is_integer(self) -> Any:
        r"""Returns True. Exists for duck type compatibility with float.is_integer."""
        ...
    def to_bytes(self, length: Any = 1, byteorder: Any = 'big', signed: Any = False) -> Any:
        r"""Return an array of bytes representing an integer.
        
          length
            Length of bytes object to use.  An OverflowError is raised if the
            integer is not representable with the given number of bytes.  Default
            is length 1.
          byteorder
            The byte order used to represent the integer.  If byteorder is 'big',
            the most significant byte is at the beginning of the byte array.  If
            byteorder is 'little', the most significant byte is at the end of the
            byte array.  To request the native byte order of the host system, use
            sys.byteorder as the byte order value.  Default is to use 'big'.
          signed
            Determines whether two's complement is used to represent the integer.
            If signed is False and a negative integer is given, an OverflowError
            is raised.
        """
        ...

class loader_input_t:
    r"""A helper class to work with linput_t related functions.
    This class is also used by file loaders scripts.
    """
    @property
    def __idc_cvt_id__(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, pycapsule: Any = None) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __swig_destroy__(self, object: Any) -> Any:
        ...
    def close(self) -> Any:
        r"""Closes the file"""
        ...
    def file2base(self, pos: int, ea1: ea_t, ea2: ea_t, patchable: bool) -> Any:
        r"""Load portion of file into the database
        This function will include (ea1..ea2) into the addressing space of the
        program (make it enabled)
        
        :param li: pointer ot input source
        :param pos: position in the file
        :param ea1: start of range of destination linear addresses
        :param ea2: end of range of destination linear addresses
        :param patchable: should the kernel remember correspondance of
                          file offsets to linear addresses.
        :returns: 1-ok,0-read error, a warning is displayed
        """
        ...
    def filename(self) -> Any:
        ...
    def from_capsule(self, pycapsule: Any) -> loader_input_t:
        ...
    def from_fp(self, fp: Any) -> Any:
        r"""A static method to construct an instance from a FILE*
        
        :param fp: a FILE pointer
        :returns: a new instance, or None
        """
        ...
    def from_linput(self, linput: linput_t) -> loader_input_t:
        ...
    def get_byte(self) -> Any:
        r"""Reads a single byte from the file. Returns None if EOF or the read byte"""
        ...
    def get_linput(self) -> linput_t:
        ...
    def gets(self, len: int) -> Any:
        r"""Reads a line from the input file. Returns the read line or None
        
        :param len: the maximum line length
        :returns: a str, or None
        """
        ...
    def getz(self, size: int, fpos: int = -1) -> Any:
        r"""Returns a zero terminated string at the given position
        
        :param size: maximum size of the string
        :param fpos: if != -1 then seek will be performed before reading
        :returns: The string or None on failure.
        """
        ...
    def open(self, filename: Any, remote: Any = False) -> Any:
        r"""Opens a file (or a remote file)
        
        :param filename: the file name
        :param remote: whether the file is local, or remote
        :returns: Boolean
        """
        ...
    def open_memory(self, start: ea_t, size: int) -> Any:
        r"""Create a linput for process memory (By internally calling idaapi.create_memory_linput())
        This linput will use dbg->read_memory() to read data
        
        :param start: starting address of the input
        :param size: size of the memory range to represent as linput
                    if unknown, may be passed as 0
        """
        ...
    def opened(self) -> Any:
        r"""Checks if the file is opened or not"""
        ...
    def read(self, size: int = -1) -> Any:
        r"""Read up to size bytes (all data if size is negative). Return an empty bytes object on EOF.
        
        :param size: the maximum number of bytes to read
        :returns: a bytes object
        """
        ...
    def readbytes(self, size: int, big_endian: bool) -> Any:
        r"""Similar to read() but it respect the endianness
        
        :param size: the maximum number of bytes to read
        :param big_endian: endianness
        :returns: a str, or None
        """
        ...
    def seek(self, offset: int, whence: Any = 0) -> Any:
        r"""Set input source position
        
        :param offset: the seek offset
        :param whence: the position to seek from
        :returns: the new position (not 0 as fseek!)
        """
        ...
    def set_linput(self, linput: Any) -> Any:
        r"""Links the current loader_input_t instance to a linput_t instance
        
        :param linput: the linput_t to link to
        """
        ...
    def size(self) -> int64:
        ...
    def tell(self) -> Any:
        r"""Returns the current position"""
        ...

class long_type:
    r"""int([x]) -> integer
    int(x, base=10) -> integer
    
    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating-point
    numbers, this truncates towards zero.
    
    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """
    denominator: getset_descriptor  # <attribute 'denominator' of 'int' objects>
    imag: getset_descriptor  # <attribute 'imag' of 'int' objects>
    numerator: getset_descriptor  # <attribute 'numerator' of 'int' objects>
    real: getset_descriptor  # <attribute 'real' of 'int' objects>
    def __abs__(self) -> Any:
        r"""abs(self)"""
        ...
    def __add__(self, value: Any) -> Any:
        r"""Return self+value."""
        ...
    def __and__(self, value: Any) -> Any:
        r"""Return self&value."""
        ...
    def __bool__(self) -> Any:
        r"""True if self else False"""
        ...
    def __ceil__(self) -> Any:
        r"""Ceiling of an Integral returns itself."""
        ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __divmod__(self, value: Any) -> Any:
        r"""Return divmod(self, value)."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __float__(self) -> Any:
        r"""float(self)"""
        ...
    def __floor__(self) -> Any:
        r"""Flooring an Integral returns itself."""
        ...
    def __floordiv__(self, value: Any) -> Any:
        r"""Return self//value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Convert to a string according to format_spec."""
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getnewargs__(self) -> Any:
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __index__(self) -> Any:
        r"""Return self converted to an integer, if self is suitable for use as an index into a list."""
        ...
    def __init__(self, args: Any, kwargs: Any) -> Any:
        r"""Initialize self.  See help(type(self)) for accurate signature."""
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __int__(self) -> Any:
        r"""int(self)"""
        ...
    def __invert__(self) -> Any:
        r"""~self"""
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lshift__(self, value: Any) -> Any:
        r"""Return self<<value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __mod__(self, value: Any) -> Any:
        r"""Return self%value."""
        ...
    def __mul__(self, value: Any) -> Any:
        r"""Return self*value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __neg__(self) -> Any:
        r"""-self"""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __or__(self, value: Any) -> Any:
        r"""Return self|value."""
        ...
    def __pos__(self) -> Any:
        r"""+self"""
        ...
    def __pow__(self, value: Any, mod: Any = None) -> Any:
        r"""Return pow(self, value, mod)."""
        ...
    def __radd__(self, value: Any) -> Any:
        r"""Return value+self."""
        ...
    def __rand__(self, value: Any) -> Any:
        r"""Return value&self."""
        ...
    def __rdivmod__(self, value: Any) -> Any:
        r"""Return divmod(value, self)."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __rfloordiv__(self, value: Any) -> Any:
        r"""Return value//self."""
        ...
    def __rlshift__(self, value: Any) -> Any:
        r"""Return value<<self."""
        ...
    def __rmod__(self, value: Any) -> Any:
        r"""Return value%self."""
        ...
    def __rmul__(self, value: Any) -> Any:
        r"""Return value*self."""
        ...
    def __ror__(self, value: Any) -> Any:
        r"""Return value|self."""
        ...
    def __round__(self, *args: Any, **kwargs: Any) -> Any:
        r"""Rounding an Integral returns itself.
        
        Rounding with an ndigits argument also returns an integer.
        """
        ...
    def __rpow__(self, value: Any, mod: Any = None) -> Any:
        r"""Return pow(value, self, mod)."""
        ...
    def __rrshift__(self, value: Any) -> Any:
        r"""Return value>>self."""
        ...
    def __rshift__(self, value: Any) -> Any:
        r"""Return self>>value."""
        ...
    def __rsub__(self, value: Any) -> Any:
        r"""Return value-self."""
        ...
    def __rtruediv__(self, value: Any) -> Any:
        r"""Return value/self."""
        ...
    def __rxor__(self, value: Any) -> Any:
        r"""Return value^self."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Returns size in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __sub__(self, value: Any) -> Any:
        r"""Return self-value."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def __truediv__(self, value: Any) -> Any:
        r"""Return self/value."""
        ...
    def __trunc__(self) -> Any:
        r"""Truncating an Integral returns itself."""
        ...
    def __xor__(self, value: Any) -> Any:
        r"""Return self^value."""
        ...
    def as_integer_ratio(self) -> Any:
        r"""Return a pair of integers, whose ratio is equal to the original int.
        
        The ratio is in lowest terms and has a positive denominator.
        
        >>> (10).as_integer_ratio()
        (10, 1)
        >>> (-10).as_integer_ratio()
        (-10, 1)
        >>> (0).as_integer_ratio()
        (0, 1)
        """
        ...
    def bit_count(self) -> Any:
        r"""Number of ones in the binary representation of the absolute value of self.
        
        Also known as the population count.
        
        >>> bin(13)
        '0b1101'
        >>> (13).bit_count()
        3
        """
        ...
    def bit_length(self) -> Any:
        r"""Number of bits necessary to represent self in binary.
        
        >>> bin(37)
        '0b100101'
        >>> (37).bit_length()
        6
        """
        ...
    def conjugate(self) -> Any:
        r"""Returns self, the complex conjugate of any int."""
        ...
    def from_bytes(self, bytes: Any, byteorder: Any = 'big', signed: Any = False) -> Any:
        r"""Return the integer represented by the given array of bytes.
        
          bytes
            Holds the array of bytes to convert.  The argument must either
            support the buffer protocol or be an iterable object producing bytes.
            Bytes and bytearray are examples of built-in objects that support the
            buffer protocol.
          byteorder
            The byte order used to represent the integer.  If byteorder is 'big',
            the most significant byte is at the beginning of the byte array.  If
            byteorder is 'little', the most significant byte is at the end of the
            byte array.  To request the native byte order of the host system, use
            sys.byteorder as the byte order value.  Default is to use 'big'.
          signed
            Indicates whether two's complement is used to represent the integer.
        """
        ...
    def is_integer(self) -> Any:
        r"""Returns True. Exists for duck type compatibility with float.is_integer."""
        ...
    def to_bytes(self, length: Any = 1, byteorder: Any = 'big', signed: Any = False) -> Any:
        r"""Return an array of bytes representing an integer.
        
          length
            Length of bytes object to use.  An OverflowError is raised if the
            integer is not representable with the given number of bytes.  Default
            is length 1.
          byteorder
            The byte order used to represent the integer.  If byteorder is 'big',
            the most significant byte is at the beginning of the byte array.  If
            byteorder is 'little', the most significant byte is at the end of the
            byte array.  To request the native byte order of the host system, use
            sys.byteorder as the byte order value.  Default is to use 'big'.
          signed
            Determines whether two's complement is used to represent the integer.
            If signed is False and a negative integer is given, an OverflowError
            is raised.
        """
        ...

class object_t:
    r"""Helper class used to initialize empty objects"""
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getitem__(self, idx: Any) -> Any:
        r"""Allow access to object attributes by index (like dictionaries)"""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, kwds: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...

class plugin_t(pyidc_opaque_object_t):
    r"""Base class for all scripted plugins."""
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any, kwargs: Any) -> Any:
        r"""Initialize self.  See help(type(self)) for accurate signature."""
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def run(self, arg: Any) -> Any:
        ...
    def term(self) -> Any:
        ...

class plugmod_t(pyidc_opaque_object_t):
    r"""Base class for all scripted multi-plugins."""
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any, kwargs: Any) -> Any:
        r"""Initialize self.  See help(type(self)) for accurate signature."""
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...

class py_clinked_object_t(pyidc_opaque_object_t):
    r"""
    This is a utility and base class for C linked objects
    
    """
    @property
    def clink(self) -> Any: ...
    @property
    def clink_ptr(self) -> Any: ...
    def __del__(self) -> Any:
        r"""Delete the link upon object destruction (only if not static)"""
        ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, lnk: Any = None) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...
    def assign(self, other: Any) -> Any:
        r"""
        Overwrite me.
        This method allows you to assign an instance contents to anothers
        :returns: Boolean
        
        """
        ...
    def copy(self) -> Any:
        r"""Returns a new copy of this class"""
        ...

class pyidc_cvt_helper__:
    r"""
    This is a special helper object that helps detect which kind
    of object is this python object wrapping and how to convert it
    back and from IDC.
    This object is characterized by its special attribute and its value
    
    """
    @property
    def value(self) -> Any: ...
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, cvt_id: Any, value: Any) -> Any:
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...

class pyidc_opaque_object_t:
    r"""This is the base class for all Python<->IDC opaque objects"""
    def __delattr__(self, name: Any) -> Any:
        r"""Implement delattr(self, name)."""
        ...
    def __dir__(self) -> Any:
        r"""Default dir() implementation."""
        ...
    def __eq__(self, value: Any) -> Any:
        r"""Return self==value."""
        ...
    def __format__(self, format_spec: Any) -> Any:
        r"""Default object formatter.
        
        Return str(self) if format_spec is empty. Raise TypeError otherwise.
        """
        ...
    def __ge__(self, value: Any) -> Any:
        r"""Return self>=value."""
        ...
    def __getattribute__(self, name: Any) -> Any:
        r"""Return getattr(self, name)."""
        ...
    def __getstate__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __gt__(self, value: Any) -> Any:
        r"""Return self>value."""
        ...
    def __hash__(self) -> Any:
        r"""Return hash(self)."""
        ...
    def __init__(self, args: Any, kwargs: Any) -> Any:
        r"""Initialize self.  See help(type(self)) for accurate signature."""
        ...
    def __init_subclass__(self) -> Any:
        r"""This method is called when a class is subclassed.
        
        The default implementation does nothing. It may be
        overridden to extend subclasses.
        
        """
        ...
    def __le__(self, value: Any) -> Any:
        r"""Return self<=value."""
        ...
    def __lt__(self, value: Any) -> Any:
        r"""Return self<value."""
        ...
    def __ne__(self, value: Any) -> Any:
        r"""Return self!=value."""
        ...
    def __new__(self, args: Any, kwargs: Any) -> Any:
        r"""Create and return a new object.  See help(type) for accurate signature."""
        ...
    def __reduce__(self) -> Any:
        r"""Helper for pickle."""
        ...
    def __reduce_ex__(self, protocol: Any) -> Any:
        r"""Helper for pickle."""
        ...
    def __repr__(self) -> Any:
        r"""Return repr(self)."""
        ...
    def __setattr__(self, name: Any, value: Any) -> Any:
        r"""Implement setattr(self, name, value)."""
        ...
    def __sizeof__(self) -> Any:
        r"""Size of object in memory, in bytes."""
        ...
    def __str__(self) -> Any:
        r"""Return str(self)."""
        ...
    def __subclasshook__(self, object: Any) -> Any:
        r"""Abstract classes can override this to customize issubclass().
        
        This is invoked early on by abc.ABCMeta.__subclasscheck__().
        It should return True, False or NotImplemented.  If it returns
        NotImplemented, the normal algorithm is used.  Otherwise, it
        overrides the normal algorithm (and the outcome is cached).
        
        """
        ...

def IDAPython_Completion(line: Any, x: Any) -> Any:
    r"""Internal utility class for auto-completion support"""
    ...

def IDAPython_ExecScript(path: Any, g: Any, print_error: Any = True) -> Any:
    r"""
    Run the specified script.
    
    This function is used by the low-level plugin code.
    
    """
    ...

def IDAPython_ExecSystem(cmd: Any) -> Any:
    r"""
    Executes a command with popen().
    
    """
    ...

def IDAPython_FormatExc(etype: Any, value: Any = None, tb: Any = None, limit: Any = None) -> Any:
    r"""
    This function is used to format an exception given the
    values returned by a PyErr_Fetch()
    
    """
    ...

def IDAPython_GetDocstrings(obj: Any) -> Any:
    ...

def IDAPython_LoadProcMod(path: Any, g: Any, print_error: Any = True) -> Any:
    r"""
    Load processor module.
    
    """
    ...

def IDAPython_UnLoadProcMod(script: Any, g: Any, print_error: Any = True) -> Any:
    r"""
    Unload processor module.
    
    """
    ...

def TRUNC(ea: Any) -> Any:
    r"""Truncate EA for the current application bitness"""
    ...

def as_UTF16(s: Any) -> Any:
    r"""Convenience function to convert a string into appropriate unicode format"""
    ...

def as_cstr(val: Any) -> Any:
    r"""
    Returns a C str from the passed value. The passed value can be of type refclass (returned by a call to buffer() or byref())
    It scans for the first \x00 and returns the string value up to that point.
    
    """
    ...

def as_int32(v: Any) -> Any:
    r"""Returns a number as a signed int32 number"""
    ...

def as_signed(v: Any, nbits: Any = 32) -> Any:
    r"""
    Returns a number as signed. The number of bits are specified by the user.
    The MSB holds the sign.
    
    """
    ...

def as_uint32(v: Any) -> Any:
    r"""Returns a number as an unsigned int32 number"""
    ...

def as_unicode(s: Any) -> Any:
    r"""Convenience function to convert a string into appropriate unicode format"""
    ...

def copy_bits(v: Any, s: Any, e: Any = -1) -> Any:
    r"""
    Copy bits from a value
    :param v: the value
    :param s: starting bit (0-based)
    :param e: ending bit
    
    """
    ...

def disable_script_timeout() -> Any:
    r"""Disables the script timeout and hides the script wait box.
    Calling L{set_script_timeout} will not have any effects until the script is compiled and executed again
    
    :returns: None
    """
    ...

def enable_extlang_python(enable: Any) -> Any:
    r"""Enables or disables Python extlang.
    When enabled, all expressions will be evaluated by Python.
    
    :param enable: Set to True to enable, False otherwise
    """
    ...

def enable_python_cli(enable: bool) -> None:
    ...

def format_basestring(_in: Any) -> str:
    ...

def notify_when(when: Any, callback: Any) -> Any:
    r"""
    Register a callback that will be called when an event happens.
    :param when: one of NW_XXXX constants
    :param callback: This callback prototype varies depending on the 'when' parameter:
                     The general callback format:
                         def notify_when_callback(nw_code)
                     In the case of NW_OPENIDB:
                         def notify_when_callback(nw_code, is_old_database)
    :returns: Boolean
    
    """
    ...

def parse_command_line3(cmdline: str) -> Any:
    ...

def pycim_get_widget(_self: Any) -> TWidget:
    ...

def pycim_view_close(_self: Any) -> None:
    ...

def pygc_create_groups(_self: Any, groups_infos: Any) -> Any:
    ...

def pygc_delete_groups(_self: Any, groups: Any, new_current: Any) -> Any:
    ...

def pygc_refresh(_self: Any) -> None:
    ...

def pygc_set_groups_visibility(_self: Any, groups: Any, expand: Any, new_current: Any) -> Any:
    ...

def replfun(func: Any) -> Any:
    ...

def require(modulename: Any, package: Any = None) -> Any:
    r"""
    Load, or reload a module.
    
    When under heavy development, a user's tool might consist of multiple
    modules. If those are imported using the standard 'import' mechanism,
    there is no guarantee that the Python implementation will re-read
    and re-evaluate the module's Python code. In fact, it usually doesn't.
    What should be done instead is 'reload()'-ing that module.
    
    This is a simple helper function that will do just that: In case the
    module doesn't exist, it 'import's it, and if it does exist,
    'reload()'s it.
    
    The importing module (i.e., the module calling require()) will have
    the loaded module bound to its globals(), under the name 'modulename'.
    (If require() is called from the command line, the importing module
    will be '__main__'.)
    
    For more information, see: <http://www.hexblog.com/?p=749>.
    
    """
    ...

def set_script_timeout(timeout: Any) -> Any:
    r"""Changes the script timeout value. The script wait box dialog will be hidden and shown again when the timeout elapses.
    See also L{disable_script_timeout}.
    
    :param timeout: This value is in seconds.
                    If this value is set to zero then the script will never timeout.
    :returns: Returns the old timeout value
    """
    ...

def struct_unpack(buffer: Any, signed: Any = False, offs: Any = 0) -> Any:
    r"""
    Unpack a buffer given its length and offset using struct.unpack_from().
    This function will know how to unpack the given buffer by using the lookup table '__struct_unpack_table'
    If the buffer is of unknown length then None is returned. Otherwise the unpacked value is returned.
    
    """
    ...

BADADDR: int  # 18446744073709551615
BADADDR32: int  # 4294967295
BADADDR64: int  # 18446744073709551615
BADSEL: int  # 18446744073709551615
HBF_CALL_WITH_NEW_EXEC: int  # 1
HBF_VOLATILE_METHOD_SET: int  # 2
NW_CLOSEIDB: int  # 2
NW_INITIDA: int  # 4
NW_OPENIDB: int  # 1
NW_REMOVE: int  # 16
NW_TERMIDA: int  # 8
PLUGIN_DBG: int  # 32
PLUGIN_DRAW: int  # 2
PLUGIN_FIX: int  # 128
PLUGIN_HIDE: int  # 16
PLUGIN_KEEP: int  # 2
PLUGIN_MOD: int  # 1
PLUGIN_MULTI: int  # 256
PLUGIN_OK: int  # 1
PLUGIN_PROC: int  # 64
PLUGIN_SEG: int  # 4
PLUGIN_SKIP: int  # 0
PLUGIN_UNL: int  # 8
PY_ICID_BYREF: int  # 1
PY_ICID_INT64: int  # 0
PY_ICID_OPAQUE: int  # 2
SEEK_CUR: int  # 1
SEEK_END: int  # 2
SEEK_SET: int  # 0
SIZE_MAX: int  # 18446744073709551615
ST_OVER_DEBUG_SEG: int  # 1
ST_OVER_LIB_FUNC: int  # 2
SWIG_PYTHON_LEGACY_BOOL: int  # 1
annotations: _Feature  # _Feature((3, 7, 0, 'beta', 1), None, 16777216)
bisect: module
builtins: module  # <module 'builtins' (built-in)>
ida_idaapi: module
inspect: module
integer_types: tuple  # (<class 'int'>,)
os: module  # <module 'os' (frozen)>
re: module
string_types: tuple  # (<class 'str'>,)
struct: module
sys: module  # <module 'sys' (built-in)>
traceback: module
weakref: module